/**
 * @function doSignup
 * @returns Promise
 */
"use strict";

const extend = require('extend'),
    apemanpromise = require('apemanpromise');

/** @lends doSignup */
function doSignup(User, Sign, data, digest, signKeys, saltLength) {
    let salt = digest.newSalt(saltLength || 24);
    return digest(data.password, salt)
        .then((hash) => {
            let user = extend(data);
            delete user.password;
            return User.create(user)
                .then((user) => {
                    let userId = user.get('id');
                    let promises = signKeys.map((signKey) => {
                        return Sign.create({
                            user_id: userId,
                            kind: signKey,
                            key: data[signKey],
                            salt: salt,
                            hash: hash,
                            signup_at: new Date()
                        });
                    });
                    return apemanpromise
                        .all(promises)
                        .then((signs) => {
                            return {
                                user: user.toJSON(),
                                signs: signs.map((sign) => {
                                    return sign.toJSON();
                                })
                            };
                        });
                });
        });
}

module.exports = doSignup;
